@@ -1,9 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -46,16 +46,15 @@ static apr_status_t shm_cleanup_owner(void *m_)
     /* name-based shared memory */
     else {
 #if APR_USE_SHMEM_MMAP_TMP
-        apr_status_t rv;
-
         if (munmap(m->base, m->realsize) == -1) {
             return errno;
         }
-        rv = apr_file_remove(m->filename, m->pool);
-        if (rv != APR_SUCCESS) {
-            return rv;
+        if (access(m->filename, F_OK)) {
+            return APR_SUCCESS;
+        }
+        else {
+            return apr_file_remove(m->filename, m->pool);
         }
-        return APR_SUCCESS;
 #endif
 #if APR_USE_SHMEM_MMAP_SHM
         if (munmap(m->base, m->realsize) == -1) {
@@ -67,22 +66,21 @@ static apr_status_t shm_cleanup_owner(void *m_)
         return APR_SUCCESS;
 #endif
 #if APR_USE_SHMEM_SHMGET
-        apr_status_t rv;
-
         /* Indicate that the segment is to be destroyed as soon
          * as all processes have detached. This also disallows any
          * new attachments to the segment. */
-        if (shmctl(m->shmid, IPC_RMID, NULL) == -1) {
+        if (shmctl(m->shmid, IPC_RMID, NULL) == -1 && errno != EINVAL) {
             return errno;
         }
         if (shmdt(m->base) == -1) {
             return errno;
         }
-        rv = apr_file_remove(m->filename, m->pool);
-        if (rv != APR_SUCCESS) {
-            return rv;
+        if (access(m->filename, F_OK)) {
+            return APR_SUCCESS;
+        }
+        else {
+            return apr_file_remove(m->filename, m->pool);
         }
-        return APR_SUCCESS;
 #endif
     }
 
@@ -118,9 +116,6 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
     if (filename == NULL) {
 #if APR_USE_SHMEM_MMAP_ZERO || APR_USE_SHMEM_MMAP_ANON
         new_m = apr_palloc(pool, sizeof(apr_shm_t));
-        if (!new_m) {
-            return APR_ENOMEM;
-        }
         new_m->pool = pool;
         new_m->reqsize = reqsize;
         new_m->realsize = reqsize + 
@@ -181,9 +176,6 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
 #if APR_USE_SHMEM_SHMGET_ANON
 
         new_m = apr_palloc(pool, sizeof(apr_shm_t));
-        if (!new_m) {
-            return APR_ENOMEM;
-        }
         new_m->pool = pool;
         new_m->reqsize = reqsize;
         new_m->realsize = reqsize;
@@ -229,9 +221,6 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
     /* Name-based shared memory */
     else {
         new_m = apr_palloc(pool, sizeof(apr_shm_t));
-        if (!new_m) {
-            return APR_ENOMEM;
-        }
         new_m->pool = pool;
         new_m->reqsize = reqsize;
         new_m->filename = apr_pstrdup(pool, filename);
@@ -276,8 +265,7 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
         }
 #endif /* APR_USE_SHMEM_MMAP_TMP */
 #if APR_USE_SHMEM_MMAP_SHM
-        /* FIXME: Is APR_OS_DEFAULT sufficient? */
-        tmpfd = shm_open(filename, O_RDWR | O_CREAT | O_EXCL, APR_OS_DEFAULT);
+        tmpfd = shm_open(filename, O_RDWR | O_CREAT | O_EXCL, 0644);
         if (tmpfd == -1) {
             return errno;
         }
@@ -299,7 +287,6 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
 
         /* FIXME: check for errors */
 
-        /* FIXME: Is it ok to close this file when using shm_open?? */
         status = apr_file_close(file);
         if (status != APR_SUCCESS) {
             return status;
@@ -378,6 +365,65 @@ APR_DECLARE(apr_status_t) apr_shm_create(apr_shm_t **m,
     return APR_ENOTIMPL;
 }
 
+APR_DECLARE(apr_status_t) apr_shm_remove(const char *filename,
+                                         apr_pool_t *pool)
+{
+#if APR_USE_SHMEM_SHMGET
+    apr_status_t status;
+    apr_file_t *file;  
+    key_t shmkey;
+    int shmid;
+#endif
+
+#if APR_USE_SHMEM_MMAP_TMP
+    return apr_file_remove(filename, pool);
+#endif
+#if APR_USE_SHMEM_MMAP_SHM
+    if (shm_unlink(filename) == -1) {
+        return errno;
+    }
+    return APR_SUCCESS;
+#endif
+#if APR_USE_SHMEM_SHMGET
+    /* Presume that the file already exists; just open for writing */    
+    status = apr_file_open(&file, filename, APR_WRITE,
+                           APR_OS_DEFAULT, pool);
+    if (status) {
+        return status;
+    }
+
+    /* ftok() (on solaris at least) requires that the file actually
+     * exist before calling ftok(). */
+    shmkey = ftok(filename, 1);
+    if (shmkey == (key_t)-1) {
+        goto shm_remove_failed;
+    }
+
+    apr_file_close(file);
+
+    if ((shmid = shmget(shmkey, 0, SHM_R | SHM_W)) < 0) {
+        goto shm_remove_failed;
+    }
+
+    /* Indicate that the segment is to be destroyed as soon
+     * as all processes have detached. This also disallows any
+     * new attachments to the segment. */
+    if (shmctl(shmid, IPC_RMID, NULL) == -1) {
+        goto shm_remove_failed;
+    }
+    return apr_file_remove(filename, pool);
+
+shm_remove_failed:
+    status = errno;
+    /* ensure the file has been removed anyway. */
+    apr_file_remove(filename, pool);
+    return status;
+#endif
+
+    /* No support for anonymous shm */
+    return APR_ENOTIMPL;
+} 
+
 APR_DECLARE(apr_status_t) apr_shm_destroy(apr_shm_t *m)
 {
     return apr_pool_cleanup_run(m->pool, m, shm_cleanup_owner);
@@ -427,9 +473,6 @@ APR_DECLARE(apr_status_t) apr_shm_attach(apr_shm_t **m,
         apr_size_t nbytes;
 
         new_m = apr_palloc(pool, sizeof(apr_shm_t));
-        if (!new_m) {
-            return APR_ENOMEM;
-        }
         new_m->pool = pool;
         new_m->filename = apr_pstrdup(pool, filename);
 
@@ -486,11 +529,7 @@ APR_DECLARE(apr_status_t) apr_shm_attach(apr_shm_t **m,
         key_t shmkey;
 
         new_m = apr_palloc(pool, sizeof(apr_shm_t));
-        if (!new_m) {
-            return APR_ENOMEM;
-        }
 
-        /* FIXME: does APR_OS_DEFAULT matter for reading? */
         status = apr_file_open(&file, filename, 
                                APR_READ, APR_OS_DEFAULT, pool);
         if (status != APR_SUCCESS) {
